import sys
import os
import re
import shutil

strDefaultFolder = "."
udwFileProcessed = 0
udwFileModified = 0
udwExpresionModified = 0

strAwkwardImport = "import com.facebook.android.*;"
strOldImport = "import com.facebook.android.R;"
strNewImport = "import com.taggames.*;\n"

strAndroidAllowed = "android."
astrForbidden = [ "facebook." ]

strTagResourceClassName = "CTagResourceHelper"
strTagResourceFunctionName = "GetDynamicResourceIDForField"
strTagResourceFunctionArrayName = "GetDynamicResourceIDsArrayForField"

def CheckJavaFiles(inFolder):

	# Check all the files from the current directory
	CheckJavaFilesRecursive(inFolder)

	# 
	print "File processed = " + str(udwFileProcessed) + " | " \
		+ "File modified = " + str(udwFileModified) + " | " \
		+ "Expressions modified = " + str(udwExpresionModified)

def CheckJavaFilesRecursive(inFolder):	
	
	for file in os.listdir(inFolder):
		# File is a directory
		if os.path.isdir(inFolder + "/" + file):
			# Recursive search
			CheckJavaFilesRecursive(inFolder + "/" + file)
		else:
			# Java file
			if IsJavaFile(file):
				CheckFileForResource(inFolder + "/" + file)
				

def CheckFileForResource(inFile):

	global udwExpresionModified
	global udwFileProcessed
	udwFileProcessed += 1
	bModified = False

	# Open the java file
	file = open(inFile, "r+")

	# Read the file
	astrFileLines = file.readlines()

	# Declare the array of string
	astrNewFileLines = []

	bImportModified = False

	# Goes through the file
	for line in astrFileLines:

		# Check for the import
		if strOldImport in line:

			# Replace with the new import
			line = strNewImport
			# bModified = True
			if not bImportModified:
				bImportModified = True
				astrNewFileLines.append(line)

			continue

		# Awkward import (might contain R.)
		if strAwkwardImport in line:

			# Check if we already added the import
			if not bImportModified:
				bImportModified = True
				astrNewFileLines.append(strNewImport)
				astrNewFileLines.append(line)
				continue

		if not bImportModified:
			astrNewFileLines.append(line)
			continue;

		# Flag for array modification
		bArray = False

		# Search the pattern R.class.resource
		mInt = re.search('R\.[^ ]*\.[^ );,+]*', line)

		# Search the pattern obtainStyledAttributes(attrs, R.class.resource
		mArrayInt = re.search('obtainStyledAttributes\(attrs, R\.[^ ]*\.[^ );,+]*', line)

		if mArrayInt != None:

			# Flag as an array
			bArray = True

		# If the expression has been found
		if mInt != None:

			# Get the found expression
			strExpression = mInt.group(0)

			# This might be allowed
			if strAndroidAllowed in line:

				bAllowed = True

				# Check forbidden expressions
				for strForbidden in astrForbidden:
					if strForbidden in line:
						# Line is forbidden, process it
						bAllowed = False
						break

				# Line is allowed
				if bAllowed:
					astrNewFileLines.append(line)
					continue

			# Separate the line and retrieve the surrounding of the R.class.resource
			astrLineParts = line.split(strExpression)

			# Split the expression into
			astrExpressionParts = strExpression.split(".")
			strClass = astrExpressionParts[1]
			strResource = astrExpressionParts[2]

			# Create the actual call we need
			strNewExpression = ""

			# Particular treatment for arrays
			if bArray:
				strNewExpression = GetNewArrayExpressionName(strClass, strResource)
			else:
				strNewExpression = GetNewExpressionName(strClass, strResource)

			# Create the new line
			line = astrLineParts[0] + strNewExpression + astrLineParts[1]

			# Mark as modified
			bModified = True
			udwExpresionModified += 1

		astrNewFileLines.append(line)

	if bModified:

		newFile = open(inFile + ".temp", "w+")

		# Save the file with the newly modified lines
		for line in astrNewFileLines:
			newFile.write(line)

		# Increase modified count if file has been modified
		global udwFileModified
		udwFileModified +=1

		# Replace old file with new
		shutil.move(newFile.name, file.name)

		# Close new file
		newFile.close()

	# Close the file
	file.close()

def GetExtension(inFile):
	aFileParts = inFile.split(".")
	return aFileParts[-1]

def IsJavaFile(inFile):
	return GetExtension(inFile) == "java"

def GetNewExpressionName(instrClass, instrResource):
	return strTagResourceClassName + "." + strTagResourceFunctionName + "(\"" + instrClass + "\", \"" + instrResource + "\")"

def GetNewArrayExpressionName(instrClass, instrResource):
	return strTagResourceClassName + "." + strTagResourceFunctionArrayName + "(\"" + instrClass + "\", \"" + instrResource + "\")"

def Main():
	strFolder = strDefaultFolder

	if len(sys.argv) == 2:

		if sys.argv[1] == "help":
			print "Usage: python TagResourceHelperRefactor [Folder]"
			print "if not specified, default folder will be the script's folder"
		else:
			strFolder = sys.argv[1]

	CheckJavaFiles(strFolder)

Main()
